
#import "YMIMBaseMessageCell.h"
#import "SLMikeSegment.h"
#import "SLMessageViewController.h"
#import "SLMikeSegmentModel.h"

@interface YMIMBaseMessageCell()

@property (nonatomic, strong) MessageModel *msgModel;

@end

@implementation YMIMBaseMessageCell

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self confignMessageView];
    }
    return self;
}

- (void)confignMessageView
{
    
    [self.msgHeaderView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(-16);
        make.top.mas_equalTo(0);
        make.width.height.mas_equalTo(52);
    }];

    [self.mikeLabelBgView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.msgHeaderView.mas_left).offset(-10);
        make.left.mas_greaterThanOrEqualTo(16);
        make.top.mas_equalTo(6);
        make.height.mas_equalTo(46);
        make.width.mas_equalTo(100);
    }];

    [self.mikeImageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.mikeLabelBgView);
        make.left.mas_equalTo(12);
    }];

    [self.mikeLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.mikeLabelBgView);
        make.left.equalTo(self.mikeImageView.mas_right);
    }];


    [self.messageLabelBgView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.msgHeaderView.mas_left).offset(-10);
        make.left.mas_greaterThanOrEqualTo(16);
        make.top.equalTo(self.mikeLabelBgView.mas_bottom).offset(8);
    }];
    [self.messageLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.top.left.mas_equalTo(10);
        make.right.bottom.mas_equalTo(-12);
    }];
    
    [self timeLabel];
}

- (void)removeAllConstViewHierarchy:(UIView *)superView
{
    for (UIView *subview in superView.subviews)
    {
        [self removeAllConstViewHierarchy:subview];
        NSArray *installedConstraints = [MASViewConstraint installedConstraintsForView:subview];
        for (MASConstraint *constraint in installedConstraints) {
            [constraint uninstall];
        }
    }
}

- (void)confignBaseMessageContentWith:(MessageModel *)msgModel {
    self.msgModel = msgModel;
    self.messageLabel.text = msgModel.sendText;
    [self removeAllConstViewHierarchy:self.contentView];
    CGFloat topTimeHeight = 0.f;
    if (msgModel.showTime) {
        NSString *timeStr = [NSString stringWithFormat:@"%f",msgModel.sendTime * 1000];
        self.timeLabel.text = [NSString timeFromFormatter:@"hh:mm" timeString:timeStr];
        topTimeHeight = 23.f;
        [self.timeLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.top.left.right.equalTo(self.contentView);
            make.height.mas_equalTo(topTimeHeight);
        }];
    }else {
        self.timeLabel.text = @"";
        [self.timeLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.top.left.right.equalTo(self.contentView);
            make.height.mas_equalTo(0);
        }];
    }
    CGFloat topHeight = msgModel.showMike ? 46 : 0;
    CGFloat topMikeBg = (topHeight ? 6 : 0) + topTimeHeight;
    if (msgModel.isReceive) {
        {//你有多少数据量
            self.messageLabel.textAlignment = NSTextAlignmentLeft;
            self.messageLabel.textColor = [UIColor blackColor];
            self.mikeLabel.textColor = [UIColor blackColor];
            self.mikeLabelBgView.backgroundColor = [UIColor whiteColor];
            self.messageLabelBgView.backgroundColor = [UIColor whiteColor];
            self.mikeImageView.image = ImageName(@"icon_msg_yy");
            self.msgHeaderView.image = ImageName(@"icon_ellipse");
            NSInteger timeLong = ceil(msgModel.sendText.length / 4.0);
            self.mikeLabel.text = [NSString stringWithFormat:@"%ld″",timeLong];
            if (!msgModel.showText) {
                [self.messageLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.top.left.equalTo(self.messageLabelBgView).offset(10);
//                    make.right.bottom.equalTo(self.messageLabelBgView).offset(-12);
                    
                }];
                [self.messageLabelBgView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.left.equalTo(self.msgHeaderView.mas_right).offset(10);
                    make.right.mas_lessThanOrEqualTo(self).offset(-16);
                    make.top.equalTo(self.mikeLabelBgView.mas_bottom).offset(8);
                    make.height.mas_lessThanOrEqualTo(0.01);
                }];
            }else {
                [self.messageLabelBgView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.left.equalTo(self.msgHeaderView.mas_right).offset(10);
                    make.right.mas_lessThanOrEqualTo(self).offset(-16);
                    make.top.equalTo(self.mikeLabelBgView.mas_bottom).offset(8);
                }];
                [self.messageLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.top.left.equalTo(self.messageLabelBgView).offset(10);
                    make.right.bottom.equalTo(self.messageLabelBgView).offset(-12);
                    
                }];
            }
        }
        {
            [self.mikeImageView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.centerY.equalTo(self.mikeLabelBgView);
                make.left.mas_equalTo(12);
            }];

            [self.mikeLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.centerY.equalTo(self.mikeLabelBgView);
                make.left.equalTo(self.mikeImageView.mas_right);
            }];
            [self.mikeLabelBgView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(self.msgHeaderView.mas_right).offset(10);
                make.right.mas_lessThanOrEqualTo(self).offset(-16);
                make.top.mas_equalTo(topMikeBg);
                make.height.mas_equalTo(topHeight);
                make.width.mas_equalTo(100);
            }];
        }
        [self.msgHeaderView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(16);
            make.top.mas_equalTo(topTimeHeight);
            make.width.height.mas_equalTo(52);
        }];
    }else {
        self.messageLabel.textAlignment = NSTextAlignmentRight;
        self.messageLabel.textColor = [UIColor whiteColor];
        self.mikeLabel.textColor = [UIColor whiteColor];
        self.mikeLabelBgView.backgroundColor = SLColor(@"#448AF7");
        self.messageLabelBgView.backgroundColor = SLColor(@"#448AF7");
        self.mikeImageView.image = ImageName(@"icon_msg_yy_w");
        NSString *headUrl = [NSString stringWithFormat:@"%@%@",[SLHttpCenter SharedInstance].serverUrl, [SLGlobalInfo SharedInstance].loginInfo.user.head];
        [self.msgHeaderView sd_setImageWithURL:[NSURL URLWithString:headUrl] placeholderImage:ImageName(@"icon_ellipse")];
        NSInteger timeLong = ceil(msgModel.audioDuration);
        self.mikeLabel.text = [NSString stringWithFormat:@"%ld″",timeLong];
        {
            if (!msgModel.showText) {
                [self.messageLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.top.left.equalTo(self.messageLabelBgView).offset(10);
//                    make.right.bottom.equalTo(self.messageLabelBgView).offset(-10);
                }];
                [self.messageLabelBgView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.right.equalTo(self.msgHeaderView.mas_left).offset(-10);
                    make.left.mas_greaterThanOrEqualTo(16);
                    make.top.equalTo(self.mikeLabelBgView.mas_bottom).offset(8);
                    make.height.mas_lessThanOrEqualTo(0.01);
                }];

            }else {
                [self.messageLabelBgView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.right.equalTo(self.msgHeaderView.mas_left).offset(-10);
                    make.left.mas_greaterThanOrEqualTo(16);
                    make.top.equalTo(self.mikeLabelBgView.mas_bottom).offset(8);
                }];
                [self.messageLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.top.left.equalTo(self.messageLabelBgView).offset(10);
                    make.right.bottom.equalTo(self.messageLabelBgView).offset(-10);
                }];
                
            }
        }
        
        {
            [self.mikeLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.centerY.equalTo(self.mikeLabelBgView);
                make.right.equalTo(self.mikeImageView.mas_left);
            }];
            [self.mikeImageView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.centerY.equalTo(self.mikeLabelBgView);
                make.right.equalTo(self.mikeLabelBgView.mas_right).offset(-12);
            }];
            [self.mikeLabelBgView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.right.equalTo(self.msgHeaderView.mas_left).offset(-10);
                make.left.mas_greaterThanOrEqualTo(self).offset(16);
                make.top.mas_equalTo(topMikeBg);
                make.height.mas_equalTo(topHeight);
                make.width.mas_equalTo(100);
            }];
        }
        {
            [self.msgHeaderView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.right.mas_equalTo(-16);
                make.top.mas_equalTo(topTimeHeight);
                make.width.height.mas_equalTo(52);
            }];
        }
    }
}

- (void)setCellLayerCornerRadiusWith:(MessageModel *)msgModel {
    [self setNeedsLayout];
    [self layoutIfNeeded];
    // Drawing code
    CGFloat cornerRadius = 12;
    //设置所需的圆角位置以及大小
    UIBezierPath *maskPath;
    UIBezierPath *maskPath1;
    if (msgModel.isReceive) {
        maskPath = [UIBezierPath bezierPathWithRoundedRect:self.messageLabelBgView.bounds byRoundingCorners:UIRectCornerTopRight | UIRectCornerBottomLeft | UIRectCornerBottomRight cornerRadii:CGSizeMake(cornerRadius , cornerRadius)];
        maskPath1 = [UIBezierPath bezierPathWithRoundedRect:self.mikeLabelBgView.bounds byRoundingCorners:UIRectCornerTopRight | UIRectCornerBottomLeft | UIRectCornerBottomRight cornerRadii:CGSizeMake(cornerRadius , cornerRadius)];
    }else {
        maskPath = [UIBezierPath bezierPathWithRoundedRect:self.messageLabelBgView.bounds byRoundingCorners:UIRectCornerTopLeft | UIRectCornerBottomLeft | UIRectCornerBottomRight cornerRadii:CGSizeMake(cornerRadius , cornerRadius)];
        maskPath1 = [UIBezierPath bezierPathWithRoundedRect:self.mikeLabelBgView.bounds byRoundingCorners:UIRectCornerTopLeft | UIRectCornerBottomLeft | UIRectCornerBottomRight cornerRadii:CGSizeMake(cornerRadius , cornerRadius)];
    }
    CAShapeLayer *maskLayer = [[CAShapeLayer alloc] init];
    maskLayer.frame = self.messageLabelBgView.bounds;
    maskLayer.path = maskPath.CGPath;
    self.messageLabelBgView.layer.mask = maskLayer;
    
    CAShapeLayer *maskLayer1 = [[CAShapeLayer alloc] init];
    maskLayer1.frame = self.mikeLabelBgView.bounds;
    maskLayer1.path = maskPath1.CGPath;
    self.mikeLabelBgView.layer.mask = maskLayer1;
}

- (void)drawRect:(CGRect)rect
{
    
}

- (void)mikeHandleTap:(UITapGestureRecognizer *)tap {
    SLMikeSegmentModel *model = [SLMikeSegmentModel initWithSegTitle:@"播放" segImage:@"icon_mike_translate" mikeType:SLMikeInputBroadcast];
    if (self.segmentBlock) {
        self.segmentBlock(model);
    }
}

- (void)mikeHandleLongPressGestures:(UILongPressGestureRecognizer *)paramSender {
    if (paramSender.state == UIGestureRecognizerStateBegan) {
        [self showMsgSegmentWith:self.mikeLabelBgView];
    }
}

- (void)msgHandleLongPressGestures:(UILongPressGestureRecognizer *)paramSender {
    if (paramSender.state == UIGestureRecognizerStateBegan) {
        [self showMsgSegmentWith:self.messageLabelBgView];
    }
}

- (void)showMsgSegmentWith:(UIView *)bgView {
    UIViewController *vc = self.viewController;
    if ([vc isKindOfClass:SLMessageViewController.class]) {
        CGRect rect = [self convertRect:bgView.frame toView:self.viewController.view];
        //加一个蒙层
        __block UIView *bView = [[UIView alloc] initWithFrame:vc.view.frame];
        bView.backgroundColor = [UIColor clearColor];
        bView.tag = SLMikeSegMentBgTag;
        [vc.view addSubview:bView];
        NSArray *arr = @[];
        if (self.msgModel.showMike && !self.msgModel.showText) {
            arr = @[
                    [SLMikeSegmentModel initWithSegTitle:@"查看" segImage:@"icon_eyes_on" mikeType:SLMikeInputLook],
                    [SLMikeSegmentModel initWithSegTitle:@"删除" segImage:@"icon_mike_delete" mikeType:SLMikeInputDelete]];
        }else if (!self.msgModel.showMike && self.msgModel.showText) {
            arr = @[
                [SLMikeSegmentModel initWithSegTitle:@"播放" segImage:@"icon_mike_translate" mikeType:SLMikeInputBroadcast],
                    [SLMikeSegmentModel initWithSegTitle:@"删除" segImage:@"icon_mike_delete" mikeType:SLMikeInputDelete]];
        }else if (self.msgModel.showMike && self.msgModel.showText) {
            if (bgView == self.mikeLabelBgView) {
                arr = @[
                        [SLMikeSegmentModel initWithSegTitle:@"删除" segImage:@"icon_mike_delete" mikeType:SLMikeInputDelete]];
            }
            if (bgView == self.messageLabelBgView) {
                arr = @[
                        [SLMikeSegmentModel initWithSegTitle:@"隐藏" segImage:@"icon_mike_delete" mikeType:SLMikeInputHidden]];
            }
        }
        CGFloat width = arr.count * 45 + 20;
        CGFloat x = self.msgModel.isReceive ? rect.origin.x : rect.origin.x + rect.size.width - width;
        SLMikeSegment *mikeSegment = [[SLMikeSegment alloc] initWithFrame:CGRectMake(x, rect.origin.y - 59, width, 50) titleArray:arr];
        WS(weakSelf);
        mikeSegment.mikeSegmentBlock = ^(SLMikeSegmentModel *model) {
            [bView removeFromSuperview];
            if (weakSelf.segmentBlock) {
                weakSelf.segmentBlock(model);
            }
        };
        [bView addSubview:mikeSegment];
        SLMessageViewController *msgVc = (SLMessageViewController *)vc;
        msgVc.hiddenMikeSegment = YES;
        UIImageView *imgView = [[UIImageView alloc] initWithImage:ImageName(@"icon_dsjx")];
        [bView addSubview:imgView];
        [imgView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerX.equalTo(mikeSegment);
            make.top.equalTo(mikeSegment.mas_bottom).offset(0);
            make.width.mas_equalTo(16);
            make.height.mas_equalTo(8);
        }];
    }
}

- (UILabel *)messageLabel
{
    if (!_messageLabel) {
        _messageLabel = [[UILabel alloc] init];
        _messageLabel.textAlignment = NSTextAlignmentRight;
        _messageLabel.numberOfLines = 0;
        _messageLabel.textColor = [UIColor whiteColor];
        _messageLabel.font = [UIFont systemFontOfSize:17];
        _messageLabel.userInteractionEnabled = YES;
        [self.messageLabelBgView addSubview:_messageLabel];
    }
    return _messageLabel;
}

- (UIView *)messageLabelBgView
{
    if (!_messageLabelBgView) {
        _messageLabelBgView = [[UIView alloc] initWithFrame:CGRectZero];
        _messageLabelBgView.backgroundColor = SLColor(@"#448AF7");
        _messageLabelBgView.clipsToBounds = YES;
        _messageLabelBgView.userInteractionEnabled = YES;
        UILongPressGestureRecognizer *gesture=[[UILongPressGestureRecognizer alloc]initWithTarget:self action:@selector(msgHandleLongPressGestures:)];
        gesture.minimumPressDuration= 0.5f;
        [_messageLabelBgView addGestureRecognizer:gesture];
        [self.contentView addSubview:_messageLabelBgView];
    }
    return _messageLabelBgView;
}
//如何学习ChatGPT开发
- (UIImageView *)msgHeaderView {
    if (!_msgHeaderView) {
        _msgHeaderView = [[UIImageView alloc] initWithFrame:CGRectZero];
        _msgHeaderView.image = ImageName(@"icon_ellipse");
        _msgHeaderView.layer.cornerRadius = 26.0f;
        _msgHeaderView.clipsToBounds = YES;
        [self.contentView addSubview:_msgHeaderView];
    }
    return _msgHeaderView;
}

- (UIView *)mikeLabelBgView {
    if (!_mikeLabelBgView) {
        _mikeLabelBgView = [[UIView alloc] initWithFrame:CGRectZero];
        _mikeLabelBgView.backgroundColor = SLColor(@"#448AF7");
        _mikeLabelBgView.clipsToBounds = YES;
        UILongPressGestureRecognizer *gesture=[[UILongPressGestureRecognizer alloc]initWithTarget:self action:@selector(mikeHandleLongPressGestures:)];
        gesture.minimumPressDuration= 0.5f;
        [_mikeLabelBgView addGestureRecognizer:gesture];
        UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(mikeHandleTap:)];
        [_mikeLabelBgView addGestureRecognizer:tap];
        [self.contentView addSubview:_mikeLabelBgView];
    }
    return _mikeLabelBgView;
}

- (UIImageView *)mikeImageView {
    if (!_mikeImageView) {
        _mikeImageView = [[UIImageView alloc] initWithFrame:CGRectZero];
        _mikeImageView.image = ImageName(@"icon_msg_yy");
        _mikeImageView.userInteractionEnabled = YES;
        [self.mikeLabelBgView addSubview:_mikeImageView];
    }
    return _mikeImageView;
}

- (UILabel *)mikeLabel
{
    if (!_mikeLabel) {
        _mikeLabel = [[UILabel alloc] init];
        _mikeLabel.text = @"30″";
        _mikeLabel.textAlignment = NSTextAlignmentRight;
        _mikeLabel.numberOfLines = 0;
        _mikeLabel.textColor = [UIColor whiteColor];
        _mikeLabel.font = [UIFont systemFontOfSize:17];
        _mikeLabel.userInteractionEnabled = YES;
        [self.mikeLabelBgView addSubview:_mikeLabel];
    }
    return _mikeLabel;
}

- (UILabel *)timeLabel {
    if (!_timeLabel) {
        _timeLabel = [[UILabel alloc] init];
        _timeLabel.textAlignment = NSTextAlignmentCenter;
        _timeLabel.textColor = [UIColor colorFormString:@"#3C3C43"];
        _timeLabel.font = [UIFont systemFontOfSize:11];
        [self.contentView addSubview:_timeLabel];
    }
    return _timeLabel;
}

@end
