//
//  SLRoleLabelViewController.m
//  SLAiELTS
//
//  Created by Gusont on 2023/2/28.
//

#import "SLRoleLabelViewController.h"
#import "SLMessageViewController.h"
#import "SLRegistViewController.h"


#import "SLPageControlSlideRound.h"
#import "SLInterestView.h"
#import "SLRoleLabelView.h"
#import "SLRoleCreatSucView.h"

#import "RoleModel.h"
#import "IQKeyboardManager.h"

@interface SLRoleLabelViewController ()

@property (nonatomic, strong) SLPageControlSlideRound *secPageControl;

@property (weak, nonatomic) IBOutlet UIButton *backBtn;
@property (weak, nonatomic) IBOutlet UIButton *nextBtn;
@property (weak, nonatomic) IBOutlet UIButton *skipBtn;
@property (weak, nonatomic) IBOutlet UIView *nameBgView;
@property (weak, nonatomic) IBOutlet UITextField *nameTextField;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *nextBtnConst;

@property (nonatomic, assign) NSInteger curSelIndex;

@property (nonatomic, strong) NSMutableArray *rolesTypes;

//views
@property (nonatomic, strong) NSMutableDictionary *roleLabelData;

@property (nonatomic, strong) UIView *roleView;

@property (nonatomic, strong) RoleModel *roleModel;
@end

@implementation SLRoleLabelViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    [self addRoleViews];
    [self.secPageControl mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(self.view);
        make.top.mas_equalTo(self.backBtn.mas_bottom).offset(16 * KScaleW);
        make.height.mas_equalTo(4);
        make.left.mas_equalTo(self.backBtn);
    }];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillChangeFrame:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardDidHide:) name:UIKeyboardWillHideNotification object:nil];
    dispatch_async(dispatch_get_main_queue(), ^{
        [self loadData];
    });
    self.navigationBar.hidden = YES;
    
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [IQKeyboardManager sharedManager].enable =NO;
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [IQKeyboardManager sharedManager].enable =YES;
}

- (void)addRoleViews {
    self.roleModel = [[RoleModel alloc] init];
    self.roleView = [[UIView alloc] initWithFrame:CGRectZero];
    self.rolesTypes = [NSMutableArray arrayWithArray:@[@(SLRoleInterset) ,@(SLRoleSex) ,@(SLRoleappearance) ,@(SLRoleLanguage), @(SLRoleName)]];
    //添加views
    [self.rolesTypes enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        if ([obj integerValue] == SLRoleInterset) {
            SLInterestView *interestView = [[SLInterestView alloc] initWithFrame:CGRectZero];
            [self.roleLabelData addEntriesFromDictionary:@{obj : interestView}];
        }else {
            SLRoleLabelView *roleLabelView = [[SLRoleLabelView alloc] initWithFrame:CGRectZero];
            [roleLabelView confignWithRoleType:[obj integerValue] :self.roleModel];
            [self.roleLabelData addEntriesFromDictionary:@{obj : roleLabelView}];
        }
    }];
    
    [self.view addSubview:self.roleView];
    [self.roleView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.secPageControl.mas_bottom).offset(2);
        make.left.right.equalTo(self.view);
        make.bottom.equalTo(self.nextBtn.mas_top);
    }];
    
    UIView *firstView = [self.roleLabelData objectForKey:@(SLRoleInterset)];
    [self.roleView addSubview:firstView];
    [firstView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.roleView);
    }];
}

- (void)loadData {
    WS(weakSelf);
    [[SLHttpCenter SharedInstance] getWithUrl:@"/api/User/GetInterestList" parameter:@{} success:^(id responseObject) {
        NSArray *interestList = [responseObject objectForKey:@"data"];
        NSArray *interestListModel = [InterestModel mj_objectArrayWithKeyValuesArray:interestList];
        SLInterestView *interestView = (SLInterestView *)[weakSelf.roleLabelData objectForKey:@(SLRoleInterset)];
        [interestView confignIntersetBtn:interestListModel];
    } failure:^(SPRequestError *error) {

    }];
}

- (IBAction)backBtnClick:(id)sender {
    [self.navigationController popViewControllerAnimated:YES];
}

- (IBAction)nextBtnClick:(id)sender {
    [self.view endEditing:YES];
    id roleModelType = [self.rolesTypes objectAtIndex:self.curSelIndex];
    if ([roleModelType integerValue] == SLRoleInterset) {
        [self saveRoleIntersets];
    }
    self.curSelIndex ++;
    self.nameBgView.hidden = YES;
    if (self.curSelIndex < self.secPageControl.count) {
        id curRoleModelType = [self.rolesTypes objectAtIndex:self.curSelIndex];
        [self.secPageControl slideRoundSelectIndex:self.curSelIndex];
        [self.roleView.subviews enumerateObjectsUsingBlock:^(__kindof UIView * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            [obj removeFromSuperview];
        }];
        UIView * view = [self.roleLabelData objectForKey:curRoleModelType];
        [self.roleView addSubview:view];
        [view mas_makeConstraints:^(MASConstraintMaker *make) {
            make.edges.equalTo(self.roleView);
        }];
    }
    if (self.curSelIndex == self.secPageControl.count - 1) {
        [self.nameTextField becomeFirstResponder];
        [self.nextBtn setTitle:@"完成" forState:UIControlStateNormal];
        self.nameBgView.hidden = NO;
        self.nameBgView.backgroundColor = [[UIColor colorFormString:@"#273F66"] colorWithAlphaComponent:0.3];
        self.nameTextField.superview.layer.backgroundColor = [[[UIColor whiteColor] colorWithAlphaComponent:0.4] CGColor];
        [self.view bringSubviewToFront:self.nextBtn];
        [self.view bringSubviewToFront:self.nameBgView];
    }
    if (self.curSelIndex == self.secPageControl.count) {
        self.roleModel.dataName = self.nameTextField.text;
        self.roleModel.userId = [SLGlobalInfo SharedInstance].loginInfo.user.userId;
        NSDictionary *dict = [self.roleModel mj_keyValues];
        [[SLHttpCenter SharedInstance] postWithUrl:@"/api/User/InsertAiData" parameter:dict success:^(id responseObject) {
            SLRoleCreatSucView *roleCreatSucView = [[SLRoleCreatSucView alloc] initWithFrame:self.view.frame];
            WS(weakSelf);
            roleCreatSucView.sucBlock = ^{
                [weakSelf.navigationController popToRootViewControllerAnimated:YES];
            };
            [roleCreatSucView alertShow];
        } failure:^(SPRequestError *error) {
            
        }];
    }
}

- (IBAction)skipBtnClick:(id)sender {
    
}

- (void)saveRoleIntersets {
    SLInterestView *interestView = [self.roleLabelData objectForKey:@(SLRoleInterset)];
    __block NSMutableString *interestStr = [NSMutableString string];
    [interestView.interestArr enumerateObjectsUsingBlock:^(InterestModel * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        if (obj.isSeleted) {
            [interestStr appendString:[NSString stringWithFormat:@"%@,",obj.interestId]];
        }
    }];
    NSString *userId = [SLGlobalInfo SharedInstance].loginInfo.user.userId ?: @"";
    [[SLHttpCenter SharedInstance] postWithUrl:@"/api/User/AddUserInterest" parameter:@{@"userId" : userId, @"interIds" : interestStr} success:^(id responseObject) {
        
    } failure:^(SPRequestError *error) {
        
    }];
}

#pragma mark --键盘弹出
- (void)keyboardWillChangeFrame:(NSNotification *)notification
{
        //取出键盘动画的时间(根据userInfo的key----UIKeyboardAnimationDurationUserInfoKey)
    CGFloat duration = [notification.userInfo[UIKeyboardAnimationDurationUserInfoKey] floatValue];

    CGRect keyboardFrame = [notification.userInfo[UIKeyboardFrameEndUserInfoKey] CGRectValue];
    //计算控制器的view需要平移的距离
//    CGFloat transformY = keyboardFrame.origin.y - CGRectGetMaxY(self.nextBtn.frame) - 10;
    //执行动画
    void (^animation)(void) = ^void(void) {
        self.nextBtnConst.constant = keyboardFrame.size.height + 15;
    };
    if (duration > 0) {
        [UIView animateWithDuration:duration * 2 animations:animation];
    } else {
        animation();
    }
}

#pragma mark --键盘收回
- (void)keyboardDidHide:(NSNotification *)notification
{
    self.nextBtnConst.constant = 100;

    CGFloat duration = [notification.userInfo[UIKeyboardAnimationDurationUserInfoKey] floatValue];
    void (^animation)(void) = ^void(void) {
        self.nextBtnConst.constant = 63;
    };
    if (duration > 0) {
        [UIView animateWithDuration:duration animations:animation];
    } else {
        animation();
    }
}

- (SLPageControlSlideRound *)secPageControl
{
    if (!_secPageControl) {
        _secPageControl = [[SLPageControlSlideRound alloc] initWithFrame:CGRectMake(24, self.view.bounds.size.height - 20, self.view.bounds.size.width - 48, 20)];
        [self.view addSubview:_secPageControl];
        _secPageControl.spaceWithRound =7.0f;
        _secPageControl.dotAlpha = 1.0;
        CGFloat w = CGRectGetWidth(_secPageControl.frame);
        NSInteger count = self.rolesTypes.count;
        _secPageControl.sizeOfRound =CGSizeMake(w / count - 1, 4.0f);
        _secPageControl.selctSizeOfRound =CGSizeMake(w / count - 1, 4.0f);
        _secPageControl.alignment = SlideRoundAlignmentLeft;
        _secPageControl.count = count;
        _secPageControl.hidden = NO;
        [_secPageControl slideRoundSelectIndex:0];
    }
    return _secPageControl;
}

- (NSMutableDictionary *)roleLabelData {
    if (!_roleLabelData) {
        _roleLabelData = [NSMutableDictionary dictionary];
    }
    return _roleLabelData;
}

@end
